/*****************************************************************************
 *   GATB : Genome Assembly Tool Box
 *   Copyright (C) 2014  INRIA
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/

#ifndef _TOOL_SRC_linker_HPP_
#define _TOOL_SRC_linker_HPP_

/********************************************************************************/
#include <gatb/gatb_core.hpp>
#include "../../../thirdparty/IteratorKmerH5/IteratorKmerH5.hpp"
#include "../../../thirdparty/quasi_dictionary/src/quasidictionary.h"
#include <algorithm>
#include <common.hpp>
#include <chrono>
/********************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// THIS FILE IS AUTOMATICALLY GENERATED...
//
// THIS IS A SIMPLE EXAMPLE HOW TO USE THE Tool CLASS. IF YOU WANT MORE FEATURES,
// YOU CAN HAVE A LOOK AT THE ToyTool SNIPPET HERE:
//
//      http://gatb-core.gforge.inria.fr/snippets_tools.html
//
////////////////////////////////////////////////////////////////////////////////

class SRC_linker : public Tool
{
	
private:
	quasidictionaryVectorKeyGeneric <IteratorKmerH5Wrapper, u_int32_t > quasiDico;
    int nbCores;
    int fingerprint_size;
    int threshold;
    int windows_size;
    bool commet_like;
	u_int64_t nbSolidKmers;
    int gamma_value;
	int kmer_size;
    int zero_density_windows_size;              // if >0: this defines a window in which the zero density (a zero is defined as a position in which no kmer is shared)  is computed
    int zero_density_threshold;                 // if zero_density_window_size >0: this threshold is used to select similar sequence having at least a window of size zero_density_window_size, in which the zero density is below zero_density_threshold
	static const size_t span = KMER_SPAN(1);
    bool keep_low_complexity;

public:

    // Constructor
	SRC_linker ();

    // Actual job done by the tool is here
    void execute ();

    void create_quasi_dictionary();

    void fill_quasi_dictionary();
    

    void parse_query_sequences();

    void load_quasi_dictionary(std::string dumped_file_name);

    void write_quasi_dictionary(std::string dumped_file_name);
};

/********************************************************************************/

#endif /* _TOOL_SRC_linker_HPP_ */
